#
# GUI Client for FreeRDP
# Version: 2.3
# Copyright (c) 2022 Vladlen Murylyov, RED SOFT.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

#!/usr/bin/python3
import subprocess
import sys

from PyQt5.QtWidgets import QMessageBox, QMainWindow, QInputDialog
from PyQt5.QtCore import pyqtSignal, QObject, Qt, QProcessEnvironment
from subprocess import Popen
from threading import Thread

from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.backends import default_backend
from base64 import urlsafe_b64encode
import json, os, re
from pathlib import Path
# Имя файла для сохранения параметров подключения

import platform
if platform.machine() == 'x86_64':
        arch = 'amd64'
elif platform.machine() == 'i686':
        arch = 'ia32'
else:
    exit(-1)

from sys import platform
SALT = b"ssJ9LgbF2X6cQ3DV"
filename = str(Path.home()) + "/.freerdp_gui/prefs.json"
filedir = str(Path.home()) + "/.freerdp_gui"
class ProcHandler(Thread, QObject):
    # Сигнал о завершении процесса
    finished = pyqtSignal(int)

    def __init__(self, proc, outputfile=""):
        super(ProcHandler, self).__init__()
        QObject.__init__(self)
        self.proc = proc
        self.outputfile = outputfile

    def run(self):
        stdoutdata, stderrdata = self.proc.communicate()
        if self.outputfile != "":
            sourceFile = open(self.outputfile, 'a')
            print("stdout data: " + str(stdoutdata.decode("utf-8").strip()), file=sourceFile)
            print("stderr data: " + str(stderrdata.decode("utf-8").strip()), file=sourceFile)
            sourceFile.close()
        else:
            print("stdout data: " + str(stdoutdata.decode("utf-8").strip()))
            print("stderr data: " + str(stderrdata.decode("utf-8").strip()))
        self.finished.emit(self.proc.returncode)


from PyQt5 import QtWidgets, uic

class Ui_MainWindow(QtWidgets.QMainWindow):
    def __init__(self):
        super(Ui_MainWindow, self).__init__()
        uic.loadUi('/usr/share/xfreerdp_gui/ui_mainWindow.ui', self)
        self.show()

class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()

        self.ui = Ui_MainWindow()
        # self.ui.setupUi(self)
        self.ui.setWindowTitle("Оболочка для FreeRDP")
        self.ui.connectButton.clicked.connect(self.connectButtonClicked)
        self.ui.saveButton.clicked.connect(self.saveButtonClicked)
        self.ui.loadButton.clicked.connect(self.loadButtonClicked)
        self.ui.deleteButton.clicked.connect(self.deleteButtonClicked)
        self.ui.prefList.doubleClicked.connect(self.loadButtonClicked)
        self.ui.fullScreenCheckBox.stateChanged.connect(self.fullScreenCheckBoxStateChange)
        self.ui.ScreenDynamicCheckBox.stateChanged.connect(self.fullScreenCheckBoxStateChange)
        self.ui.startApplicationCheckBox.stateChanged.connect(self.startApplicationCheckBoxStateChange)

        self.ui.connectPrinterCheckBox.stateChanged.connect(self.printerCheckBoxStateChange)
        self.ui.connectTokenCheckBox.stateChanged.connect(self.tokenCheckBoxStateChange)
        self.ui.choose_secureCheckBox.stateChanged.connect(self.secureCheckBoxStateChange)
        self.ui.ScreenAdditionalCheckBox.stateChanged.connect(self.ScreenAdditionalCheckBoxStateChange)
        self.ui.useLogsCheckBox.stateChanged.connect(self.useLogsCheckBoxStateChange)
        self.ui.ScreenDynamicCheckBox.stateChanged.connect(self.ScreenDynamicCheckBoxStateChange)


        self.screenResWidths = [640, 800, 1024, 1280, 1152, 1280, 1360, 1280, 1440, 1280, 1400, 1680, 1600, 1920, 1792,
                                1856, 1920, 2560]
        self.screenResHeights = [480, 600, 768, 768, 864, 800, 768, 960, 900, 1024, 1050, 1050, 1200, 1200, 1344, 1392,
                                 1440, 1600]

        for i in range(0, len(self.screenResWidths)):
            self.ui.screenResComboBox.addItem(str(self.screenResWidths[i]) + "x" + str(self.screenResHeights[i]))

        self.colordepth = [32, 24, 16, 8]
        for i in range(0, len(self.colordepth)):
            self.ui.colordepthComboBox.addItem(str(self.colordepth[i]))

        self.jpegpercent = [100, 75, 50]
        for i in range(0, len(self.jpegpercent)):
            self.ui.qualityComboBox.addItem(str(self.jpegpercent[i]))

        self.loglevel = ["INFO", "FATAL", "ERROR", "WARN", "DEBUG", "TRACE"]
        for i in range(0, len(self.loglevel)):
            self.ui.loglevelComboBox.addItem(str(self.loglevel[i]))

        index = self.ui.screenResComboBox.findText("1024x768", Qt.MatchFixedString)
        if index >= 0:
            self.ui.screenResComboBox.setCurrentIndex(index)

        p_list = self.get_printers()
        if p_list:
            for i in range(0, len(p_list)):
                self.ui.PrinterscomboBox.addItem(str(p_list[i]))

        token_list = self.get_tokens()
        if token_list:
            for i in range(0, len(token_list)):
                self.ui.TokenscomboBox.addItem(str(token_list[i]))

        secure_list = ["ext", "nla", "rdp", "tls"]
        for i in range(0, len(secure_list)):
            self.ui.secureComboBox.addItem(str(secure_list[i]))

        index = self.ui.secureComboBox.findText("nla", Qt.MatchFixedString)
        if index >= 0:
            self.ui.secureComboBox.setCurrentIndex(index)

        self.ui.cert_ignoreCheckBox.setChecked(True)

        self.ui.PrinterscomboBox.setEnabled(False)
        self.ui.TokenscomboBox.setEnabled(False)
        self.ui.secureComboBox.setEnabled(False)
        self.ui.ScreenDynamicCheckBox.setEnabled(False)
        self.ui.DisconnectWallpapersCheckBox.setEnabled(False)

        self.ui.appNameLineEdit.setEnabled(False)

        self.ui.colordepthComboBox.setEnabled(False)
        self.ui.qualityComboBox.setEnabled(False)

        self.ui.logfilenameLineEdit.setEnabled(False)
        self.ui.loglevelComboBox.setEnabled(False)

        self.ui.folderNameLineEdit.setEnabled(False)
        self.ui.folderPathLineEdit.setEnabled(False)

        self.ui.connectFolderCheckBox.stateChanged.connect(self.connectFolderCheckBoxStateChange)

        self.data = {}
        # Чтение сохраненнёых параметров из файла
        if not os.path.exists(f"{filedir}"):
            os.system(f"mkdir {filedir}")

        if os.path.exists(f"{filename}"):
            with open(f"{filename}") as json_file:
                text = json_file.read()
                self.data = json.loads(text)
                for key in self.data:
                    self.ui.prefList.addItem(key)
        else:
            os.system("echo {} > "+f"{filename}")

    def get_tokens(self):
        if os.path.exists(f"/opt/cprocsp/bin/{arch}/csptest"):
            tokens = os.popen(f"/opt/cprocsp/bin/{arch}/csptest -card -enum").readlines()
            token_list = []
            if tokens[-1] == "[ErrorCode: 0x00000000]\n":
                for token in tokens[:-2]:
                    token_list.append(token.strip())
                return token_list

    def startApplicationCheckBoxStateChange(self, a0):
        if self.ui.startApplicationCheckBox.isChecked():
            self.ui.appNameLineEdit.setEnabled(True)
        else:
            self.ui.appNameLineEdit.setEnabled(False)

    def ScreenDynamicCheckBoxStateChange(self, a0):
        if self.ui.ScreenDynamicCheckBox.isChecked():
            self.ui.screenResComboBox.setEnabled(False)
            self.ui.fullScreenCheckBox.setEnabled(False)
        else:
            self.ui.screenResComboBox.setEnabled(True)
            self.ui.fullScreenCheckBox.setEnabled(True)

    def fullScreenCheckBoxStateChange(self, a0):
        if self.ui.fullScreenCheckBox.isChecked():
            self.ui.screenResComboBox.setEnabled(False)
        else:
            self.ui.screenResComboBox.setEnabled(True)

    def tokenCheckBoxStateChange(self, a0):
        if self.ui.connectTokenCheckBox.isChecked():
            self.ui.TokenscomboBox.setEnabled(True)
        else:
            self.ui.TokenscomboBox.setEnabled(False)

    def connectFolderCheckBoxStateChange(self, a0):
        if self.ui.connectFolderCheckBox.isChecked():
            self.ui.folderNameLineEdit.setEnabled(True)
            self.ui.folderPathLineEdit.setEnabled(True)
        else:
            self.ui.folderNameLineEdit.setEnabled(False)
            self.ui.folderPathLineEdit.setEnabled(False)

    def printerCheckBoxStateChange(self, a0):
        if self.ui.connectPrinterCheckBox.isChecked():
            self.ui.PrinterscomboBox.setEnabled(True)
        else:
            self.ui.PrinterscomboBox.setEnabled(False)

    def secureCheckBoxStateChange(self, a0):
        if self.ui.choose_secureCheckBox.isChecked():
            self.ui.secureComboBox.setEnabled(True)
        else:
            self.ui.secureComboBox.setEnabled(False)

    def ScreenAdditionalCheckBoxStateChange(self, a0):
        if self.ui.ScreenAdditionalCheckBox.isChecked():
            self.ui.colordepthComboBox.setEnabled(True)
            self.ui.qualityComboBox.setEnabled(True)
            self.ui.ScreenDynamicCheckBox.setEnabled(True)
            self.ui.DisconnectWallpapersCheckBox.setEnabled(True)

        else:
            self.ui.colordepthComboBox.setEnabled(False)
            self.ui.qualityComboBox.setEnabled(False)
            self.ui.ScreenDynamicCheckBox.setEnabled(False)
            self.ui.DisconnectWallpapersCheckBox.setEnabled(False)

    def useLogsCheckBoxStateChange(self, a0):
        if self.ui.useLogsCheckBox.isChecked():
            self.ui.logfilenameLineEdit.setEnabled(True)
            self.ui.loglevelComboBox.setEnabled(True)
        else:
            self.ui.logfilenameLineEdit.setEnabled(False)
            self.ui.loglevelComboBox.setEnabled(False)

    def deleteButtonClicked(self):
        if len(self.ui.prefList.selectedItems()) == 0:
            msgBox = QMessageBox()
            msgBox.setText("Выберите элемент списка")
            msgBox.setWindowTitle("Удаление параметров")
            msgBox.setIcon(QMessageBox.Icon.Information)
            msgBox.setStandardButtons(QMessageBox.StandardButton.Ok)
            msgBox.exec()
        else:
            key = self.ui.prefList.selectedItems()[0].text()
            self.ui.prefList.takeItem(self.ui.prefList.row(self.ui.prefList.selectedItems()[0]))
            self.data.pop(key)
            with open(f"{filename}", 'w') as f:
                json.dump(self.data, f)

    def loadButtonClicked(self):
        if len(self.ui.prefList.selectedItems()) == 0:
            msgBox = QMessageBox()
            msgBox.setText("Выберите элемент списка")
            msgBox.setWindowTitle("Загрузка параметров")
            msgBox.setIcon(QMessageBox.Icon.Information)
            msgBox.setStandardButtons(QMessageBox.StandardButton.Ok)
            msgBox.exec()
        else:
            name = self.ui.prefList.selectedItems()[0].text()
            self.ui.ipAddressLineEdit.setText(self.data[name][0])
            self.ui.userLineEdit.setText(self.data[name][1])
            self.ui.domainLineEdit.setText(self.data[name][2])

            # Ключ для дешифрования пароля
            key = (name + self.ui.ipAddressLineEdit.text()).encode()
            kdf = PBKDF2HMAC(
                algorithm=hashes.SHA256(),
                length=32,
                salt=SALT,
                iterations=1000,
                backend=default_backend()
            )
            fernet_key = urlsafe_b64encode(kdf.derive(key))

            # Дешифрование пароля
            f = Fernet(fernet_key)

            decrypted_password = f.decrypt(self.data[name][3].encode('utf-8')).decode('utf-8')

            self.ui.passwordLineEdit.setText(decrypted_password)
            self.ui.screenResComboBox.setCurrentText(self.data[name][4])

            if self.data[name][5] == 0:
                self.ui.fullScreenCheckBox.setChecked(False)
            else:
                self.ui.fullScreenCheckBox.setChecked(True)

            if self.data[name][6] == 0:
                self.ui.connectFolderCheckBox.setChecked(False)
            else:
                self.ui.connectFolderCheckBox.setChecked(True)

            self.ui.folderNameLineEdit.setText(self.data[name][7])
            self.ui.folderPathLineEdit.setText(self.data[name][8])

            if self.data[name][9] == 0:
                self.ui.connectPrinterCheckBox.setChecked(False)
            else:
                self.ui.connectPrinterCheckBox.setChecked(True)

            index = self.ui.PrinterscomboBox.findText(self.data[name][10], Qt.MatchFixedString)
            if index >= 0:
                self.ui.PrinterscomboBox.setCurrentIndex(index)

            if self.data[name][11] == 0:
                self.ui.soundForwardCheckBox.setChecked(False)
            else:
                self.ui.soundForwardCheckBox.setChecked(True)

            if self.data[name][12] == 0:
                self.ui.connectTokenCheckBox.setChecked(False)
            else:
                self.ui.connectTokenCheckBox.setChecked(True)

            index = self.ui.TokenscomboBox.findText(self.data[name][13], Qt.MatchFixedString)
            if index >= 0:
                self.ui.TokenscomboBox.setCurrentIndex(index)

            if self.data[name][14] == 0:
                self.ui.cert_ignoreCheckBox.setChecked(False)
            else:
                self.ui.cert_ignoreCheckBox.setChecked(True)

            self.ui.titleLineEdit.setText(self.data[name][15])

            if self.data[name][16] == 0:
                self.ui.choose_secureCheckBox.setChecked(False)
            else:
                self.ui.choose_secureCheckBox.setChecked(True)

            index = self.ui.secureComboBox.findText(self.data[name][17], Qt.MatchFixedString)
            if index >= 0:
                self.ui.secureComboBox.setCurrentIndex(index)

            if self.data[name][18] == 0:
                self.ui.ScreenAdditionalCheckBox.setChecked(False)
            else:
                self.ui.ScreenAdditionalCheckBox.setChecked(True)

            index = self.ui.colordepthComboBox.findText(self.data[name][19], Qt.MatchFixedString)
            if index >= 0:
                self.ui.colordepthComboBox.setCurrentIndex(index)

            index = self.ui.qualityComboBox.findText(self.data[name][20], Qt.MatchFixedString)
            if index >= 0:
                self.ui.qualityComboBox.setCurrentIndex(index)

            if self.data[name][21] == 0:
                self.ui.useLogsCheckBox.setChecked(False)
            else:
                self.ui.useLogsCheckBox.setChecked(True)

            self.ui.logfilenameLineEdit.setText(self.data[name][22])

            index = self.ui.loglevelComboBox.findText(self.data[name][23], Qt.MatchFixedString)
            if index >= 0:
                self.ui.loglevelComboBox.setCurrentIndex(index)

            for i in range(self.ui.prefList.count()):
                item = self.ui.prefList.item(i)
                # self.ui.prefList.setItemSelected(item, False)
                item.setSelected(False)

            # print(len(self.data[name]))
            if len(self.data[name]) > 24:
                if self.data[name][24] == 0:
                    self.ui.reconnectCheckBox.setChecked(False)
                else:
                    self.ui.reconnectCheckBox.setChecked(True)
                if self.data[name][25] == 0:
                    self.ui.ScreenDynamicCheckBox.setChecked(False)
                else:
                    self.ui.ScreenDynamicCheckBox.setChecked(True)
                if self.data[name][26] == 0:
                    self.ui.DisconnectWallpapersCheckBox.setChecked(False)
                else:
                    self.ui.DisconnectWallpapersCheckBox.setChecked(True)
                if self.data[name][27] == 0:
                    self.ui.startApplicationCheckBox.setChecked(False)
                else:
                    self.ui.startApplicationCheckBox.setChecked(True)
                self.ui.appNameLineEdit.setText(self.data[name][28])

    def saveButtonClicked(self):
        ipaddress = self.ui.ipAddressLineEdit.text()
        user = self.ui.userLineEdit.text()
        password = self.ui.passwordLineEdit.text()

        full_screen = 1 if self.ui.fullScreenCheckBox.isChecked() else 0
        connect_folder = 1 if self.ui.connectFolderCheckBox.isChecked() else 0
        connect_printer = 1 if self.ui.connectPrinterCheckBox.isChecked() else 0
        sound_forward = 1 if self.ui.soundForwardCheckBox.isChecked() else 0
        connect_token = 1 if self.ui.connectTokenCheckBox.isChecked() else 0
        cert_ignore = 1 if self.ui.cert_ignoreCheckBox.isChecked() else 0
        use_secure = 1 if self.ui.choose_secureCheckBox.isChecked() else 0
        ScreenAdditional = 1 if self.ui.ScreenAdditionalCheckBox.isChecked() else 0
        useLogsCheckBox = 1 if self.ui.useLogsCheckBox.isChecked() else 0
        useAutoReconenct = 1 if self.ui.reconnectCheckBox.isChecked() else 0
        useScreenDynamic = 1 if self.ui.ScreenDynamicCheckBox.isChecked() else 0
        useDisconnectWallpapers = 1 if self.ui.DisconnectWallpapersCheckBox.isChecked() else 0
        usestartApplication = 1 if self.ui.startApplicationCheckBox.isChecked() else 0

        if len(self.ui.prefList.selectedItems()) != 0:
            for cons in self.data:
                if self.ui.prefList.selectedItems()[0].text() == cons:
                    if self.check_min_input(ipaddress, user, password):
                        self.data[cons] = [
                            self.ui.ipAddressLineEdit.text(), #0
                            self.ui.userLineEdit.text(), #1
                            self.ui.domainLineEdit.text(), #2
                            self.crypt_password(cons).decode("utf-8"), #3
                            self.ui.screenResComboBox.currentText(), #4
                            full_screen, #5
                            connect_folder, #6
                            self.ui.folderNameLineEdit.text(), #7
                            self.ui.folderPathLineEdit.text(), #8
                            connect_printer, #9
                            self.ui.PrinterscomboBox.currentText(), #10
                            sound_forward, #11
                            connect_token, #12
                            self.ui.TokenscomboBox.currentText(), #13
                            cert_ignore, #14
                            self.ui.titleLineEdit.text(), #15
                            use_secure, #16
                            self.ui.secureComboBox.currentText(), #17
                            ScreenAdditional, #18
                            self.ui.colordepthComboBox.currentText(), #19
                            self.ui.qualityComboBox.currentText(), #20
                            useLogsCheckBox, #21
                            self.ui.logfilenameLineEdit.text(), #22
                            self.ui.loglevelComboBox.currentText(), #23
                            useAutoReconenct,  # 24
                            useScreenDynamic,  # 25
                            useDisconnectWallpapers,  # 26
                            usestartApplication,  # 27
                            self.ui.appNameLineEdit.text()  # 28
                        ]
                    with open(f"{filename}", 'w') as f:
                        json.dump(self.data, f)
        else:
            name, ok = QInputDialog.getText(self, 'Ввод данных', 'Введите имя для сохраняемых параметров')
            if ok:
                if name in self.data:
                    msgBox = QMessageBox()
                    msgBox.setText("Ошибка. Данное имя уже существует")
                    msgBox.setWindowTitle("Сохранение параметров")
                    msgBox.setIcon(QMessageBox.Icon.Critical)
                    msgBox.setStandardButtons(QMessageBox.StandardButton.Ok)
                    msgBox.exec()
                else:
                    self.data[name] = [
                        self.ui.ipAddressLineEdit.text(),  # 0
                        self.ui.userLineEdit.text(),  # 1
                        self.ui.domainLineEdit.text(),  # 2
                        self.crypt_password(name).decode("utf-8"),  # 3
                        self.ui.screenResComboBox.currentText(),  # 4
                        full_screen,  # 5
                        connect_folder,  # 6
                        self.ui.folderNameLineEdit.text(),  # 7
                        self.ui.folderPathLineEdit.text(),  # 8
                        connect_printer,  # 9
                        self.ui.PrinterscomboBox.currentText(),  # 10
                        sound_forward,  # 11
                        connect_token,  # 12
                        self.ui.TokenscomboBox.currentText(),  # 13
                        cert_ignore,  # 14
                        self.ui.titleLineEdit.text(),  # 15
                        use_secure,  # 16
                        self.ui.secureComboBox.currentText(),  # 17
                        ScreenAdditional,  # 18
                        self.ui.colordepthComboBox.currentText(),  # 19
                        self.ui.qualityComboBox.currentText(),  # 20
                        useLogsCheckBox,  # 21
                        self.ui.logfilenameLineEdit.text(),  # 22
                        self.ui.loglevelComboBox.currentText(),  # 23
                        useAutoReconenct, # 24
                        useScreenDynamic, # 25
                        useDisconnectWallpapers, # 26
                        usestartApplication, # 27
                        self.ui.appNameLineEdit.text() # 28
                    ]
                    with open(f"{filename}", 'w') as f:
                        json.dump(self.data, f)
                    self.ui.prefList.addItem(name)
        for i in range(self.ui.prefList.count()):
            item = self.ui.prefList.item(i)
            # self.ui.prefList.setItemSelected(item, False)
            item.setSelected(False)

    def crypt_password(self, name):
        key = (name + self.ui.ipAddressLineEdit.text()).encode()
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=SALT,
            iterations=1000,
            backend=default_backend()
        )
        fernet_key = urlsafe_b64encode(kdf.derive(key))

        # Шифрование пароля
        f = Fernet(fernet_key)
        encrypted_password = f.encrypt(self.ui.passwordLineEdit.text().encode('utf-8'))
        return encrypted_password

    def check_min_input(self, ipaddress, user, password):
        if ipaddress == "":
            msgBox = QMessageBox()
            msgBox.setWindowTitle("Ошибка")
            msgBox.setText("Введите ip-адрес")
            msgBox.setIcon(QMessageBox.Icon.Warning)
            msgBox.setStandardButtons(QMessageBox.StandardButton.Ok)
            msgBox.exec()
            return
        elif user == "":
            msgBox = QMessageBox()
            msgBox.setWindowTitle("Ошибка")
            msgBox.setText("Введите имя пользователя")
            msgBox.setIcon(QMessageBox.Icon.Warning)
            msgBox.setStandardButtons(QMessageBox.StandardButton.Ok)
            msgBox.exec()
            return
        elif password == "":
            msgBox = QMessageBox()
            msgBox.setWindowTitle("Ошибка")
            msgBox.setText("Введите пароль")
            msgBox.setIcon(QMessageBox.Icon.Warning)
            msgBox.setStandardButtons(QMessageBox.StandardButton.Ok)
            msgBox.exec()
            return
        return True

    def connectButtonClicked(self):
        ipaddress = self.ui.ipAddressLineEdit.text()
        user = self.ui.userLineEdit.text()
        password = self.ui.passwordLineEdit.text()
        # password = password.translate({i: f'\\' + chr(i) for i in b"""!"#$%&\'()*+,-./:;<=>?@[]^_`{|}~"""})
        domain = self.ui.domainLineEdit.text()

        if self.check_min_input(ipaddress, user, password):
            # Аргументы для запуска xfreerdp
            args = []

            if platform == "linux" or platform == "linux2":
                args.append("xfreerdp")
            else:
                return

            if self.ui.portLineEdit.text() != "":
                args.append("/v:" + ipaddress + ":" + str(self.ui.portLineEdit.text()))
            else:
                args.append("/v:" + ipaddress)
            args.append("/u:" + user)
            if domain != "":
                args.append("/d:" + domain)
            args.append("/p:" + password)

            if self.ui.ScreenDynamicCheckBox.isChecked() and self.ui.ScreenAdditionalCheckBox.isChecked():
                args.append("/dynamic-resolution")
            elif self.ui.fullScreenCheckBox.isChecked():
                args.append("/f")
            else:
                width = self.screenResWidths[self.ui.screenResComboBox.currentIndex()]
                height = self.screenResHeights[self.ui.screenResComboBox.currentIndex()]
                args.append("/w:" + str(width))
                args.append("/h:" + str(height))
            if self.ui.ScreenAdditionalCheckBox.isChecked():

                self.colordepth = [32, 24, 16, 8]

                depth = self.colordepth[self.ui.colordepthComboBox.currentIndex()]
                quality = self.jpegpercent[self.ui.qualityComboBox.currentIndex()]
                args.append("/bpp:" + str(depth))
                args.append("/jpeg")
                args.append("/jpeg-quality:" + str(quality))
                if self.ui.DisconnectWallpapersCheckBox.isChecked():
                    args.append("-wallpaper")
            flag_cert = True
            end = ""
            if self.ui.cert_ignoreCheckBox.isChecked():
                args.append("/cert:ignore")
                flag_cert = False
            else:
                end = "<<< 'Y'"
            if self.ui.connectFolderCheckBox.isChecked():
                args.append("/drive:" + self.ui.folderNameLineEdit.text() + "," + self.ui.folderPathLineEdit.text())

            if self.ui.soundForwardCheckBox.isChecked():
                args.append("/sound:sys:alsa")
                args.append("/sound:sys:oss,dev:1,format:1")
                args.append("/microphone:sys:alsa")
                args.append("/microphone:sys:oss,dev:1,format:1")

            if self.ui.connectTokenCheckBox.isChecked():
                if self.ui.TokenscomboBox.currentText() == "" or not os.path.exists(os.path.exists(f"/opt/cprocsp/bin/{arch}/csptest")):
                    msgBox = QMessageBox()
                    msgBox.setText("Токен не обнаружен или не установлено СКЗИ КриптоПро")
                    msgBox.setWindowTitle("Ошибка")
                    msgBox.setIcon(QMessageBox.Icon.Critical)
                    msgBox.setStandardButtons(QMessageBox.StandardButton.Ok)
                    msgBox.exec()
                    return
                args.append("/smartcard:" + f"\"{self.ui.TokenscomboBox.currentText()}\"")

            if self.ui.redirectHomeFolderCheckBox.isChecked():
                args.append("+home-drive")

            if self.ui.startApplicationCheckBox.isChecked():
                if self.ui.appNameLineEdit.text() != "":
                    args.append(f"/app:\"||{self.ui.appNameLineEdit.text()}\"")

            if self.ui.connectPrinterCheckBox.isChecked():
                args.append("/a:printer," + self.ui.PrinterscomboBox.currentText())

            title = self.ui.titleLineEdit.text()
            if title != "":
                args.append("/t:" + title)


            if self.ui.choose_secureCheckBox.isChecked():
                args.append("/sec:" + self.ui.secureComboBox.currentText())

            if self.ui.useLogsCheckBox.isChecked():
                args.append(f"/log-level:{str(self.loglevel[self.ui.loglevelComboBox.currentIndex()])}")
            if end != "":
                args.append(end)
            # Запуск процесса для программы xfreerdp
            outfile = ""
            if self.ui.useLogsCheckBox.isChecked():
                if not os.path.isdir("/tmp/xfreerdp"):
                    os.system("mkdir -p /tmp/xfreerdp")
                if str(self.ui.logfilenameLineEdit.text()) != "":
                    outfile = str(self.ui.logfilenameLineEdit.text())
                else:
                    outfile = "xfreerdp.log"
                outfile = f"/tmp/xfreerdp/{outfile}"
            arg_str = " ".join(str(arg) for arg in args)
            print(arg_str)

            proc = Popen(arg_str, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)

            # Создание потока, который будет ожидать завершение процесса proc
            # и возвращать код возврата
            procHandler = ProcHandler(proc, outfile)
            procHandler.finished.connect(self.procFinished)
            procHandler.start()

    def get_printers(self):
        printers = os.popen("lpstat -p").readlines()
        printers_list = []
        for p in printers:
            p = p.strip()
            p = re.sub(r"printer |принтер ", "", p)
            p = re.sub(r" is idle\..*| свободен\..*", "", p)
            printers_list.append(p)
        return printers_list

    # Вызывается при завершении процесса xfreerdp
    def procFinished(self, returnCode):
        if platform == "linux" or platform == "linux2":
            if returnCode == 0 or returnCode == 12:
                return
        elif platform == "win32":
            if returnCode == 0 or returnCode == 65548:
                return

        msgBox = QMessageBox()
        msgBox.setText("Ошибка при подключении\nПроверьте правильность ввода параметров")
        msgBox.setWindowTitle("Ошибка")
        msgBox.setIcon(QMessageBox.Icon.Critical)
        msgBox.setStandardButtons(QMessageBox.StandardButton.Ok)
        msgBox.exec()

def main():
    app = QtWidgets.QApplication(sys.argv)
    mainWindow = MainWindow()
    sys.exit(app.exec())

if __name__ == "__main__":
    main()


